var goals = require('hivemind_goals')
var SetupOverlord = require('overlords_setup_overlord');
const roles = require('creeps_role_map');

var hivemind = {

    /** List of references to all overlord objects */
    overlords: [],

    /** Lets the hivemind run. That means at the start initializing the hivemind's memory or advancing its state*/
    run: function () {
        this.overlords = [];  // fixing issue where the next tick still had this list when the script kept running
        if (Memory.last_tick === undefined) {
            hivemind.initialize_memory();
        }
        else {
            //console.log("Hivemind: Loading all overlords ...");
            hivemind.deserialize_overlords();

            //console.log("Hivemind: Advancing state ...");
            hivemind.advance_state();

            //console.log("Hivemind: Saving all overlords ...");
            hivemind.serialize_overlords();
        }
    },

    /** Initializes memory variables. Expected to be only called once at the start of the entire script. */
    initialize_memory: function () {
        console.log("Initializing memory...")
        Memory.last_tick = 0;
        Memory.overlords = {};
        
        var init_overlord = this.create_new_overlord("Adam", goals.INITIAL_SETUP);

        for(var curr_spawn_name in Game.spawns) {  // at the very start each spawn (the only spawn) needs to get assigned to the only overlord
            init_overlord.add_spawn(curr_spawn_name);
        }

        //console.log("Hivemind: Saving all overlords ...");
        this.serialize_overlords();
    },

    /** Serializes all overlords into memory */
    create_new_overlord: function (name, goal) {
        console.log("Creating overlord \""+name+"\" with goal "+goal+"...");
        if (goal === goals.INITIAL_SETUP) {
            var new_overlord = new SetupOverlord(name);
        }
        else {
            throw "Creating overlord with unknown goal!"
        }
        new_overlord.goal = goal;
        this.overlords.push(new_overlord);

        return new_overlord;
    },

    /** Serializes all overlords into memory */
    serialize_overlords: function () {
        for (let overlord of this.overlords) {
            overlord.serialize();
        }
    },

    /** Deserializes all overlords from memory */
    deserialize_overlords: function () {
        for (var overlord_id in Memory.overlords) {
            if (Memory.overlords[overlord_id]['goal'] === goals.INITIAL_SETUP) {
                var overlord = new SetupOverlord(overlord_id, goals.INITIAL_SETUP) // TODO fix loading correct overlord type
            }
            else {
                throw "Tried to deserialize overlord with unknown goal"
            }
            overlord.deserialize();
            
            this.overlords.push(overlord);
        }
    },

    /** Starts querying global info Updates the hivemind state to decide on goals */
    advance_state: function () {
        if (Memory.last_tick === 0) { // last_tick is used as a makeshift timestamp to remember if something was this tick or the last.
            Memory.last_tick = 1;
            //console.log("Tick is now 0");
        }
        else {
            Memory.last_tick = 0;
            //console.log("Tick is now 1");
        }

        for (let overlord of this.overlords) {
            //console.log("Hivemind: Running overlord \""+overlord.id+"\" now ...");
            overlord.run();
        }
        
        for (let name in Game.creeps) {
            let creep = Game.creeps[name];
            let creep_role = creep.memory.role;
            console.log("Hivemind: Running creep \""+name+"|"+creep_role+"\"");
            let role = _.filter(roles, function(node) {return node.ident === creep_role})[0];
            if (role === undefined) {
                var blk_str = [];
                _.each(roles, function(node) {
                    var harvester_obj = require('creeps_harvester');
                    var str = node+": "+node.ident+"|"+Object.keys(harvester_obj);
                    blk_str.push(str);
                });
                console.log("[ERROR] A creep has a role that is unknown. Creep Name: ", name, "Creep Role:", creep_role, "Known Roles: ", blk_str.join(", "));
                throw "Unknown role";
            }
            role.obj.run(creep);
        }
    },
};

module.exports = hivemind;
